<?php
/*
Plugin Name: Send code woo
Plugin URI: https://pluginstech.com
Description: Este plugin avançado para WooCommerce permite associar uma sequência de informações, como códigos ou números de série, a um produto. Quando uma venda é realizada, o plugin envia automaticamente o primeiro item da lista associada ao produto vendido. A lista de informações deve ser inserida com os itens separados por vírgulas (ex: 1CODE, 2CODE, 3CODE, etc.). Após o envio, o item utilizado é removido da lista, garantindo que o próximo item seja enviado na próxima venda. O plugin também envia um webhook com os detalhes da compra e oferece integração com a Pluga API, permitindo o envio do código via WhatsApp para seus clientes, se você tiver um plano ativo. Dados dinâmicos da mensagem: {nome_cliente}, {email_cliente}, {itens_pedido}, {valor_total}, esse aqui é para informar o código {codes}
Version: 1.0
Author: plugins tech
Author URI: https://pluginstech.com
License: 
License URI: 
*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

class WooCommerce_Combined_Plugin {

    public static function init() {
        if (class_exists('WooCommerce')) {
            add_action('woocommerce_product_options_general_product_data', [__CLASS__, 'add_fields']);
            add_action('woocommerce_process_product_meta', [__CLASS__, 'save_fields']);
            add_action('woocommerce_order_status_completed', [__CLASS__, 'process_order']);
            add_action('woocommerce_order_status_processing', [__CLASS__, 'process_order']);
            add_action('woocommerce_order_status_failed', [__CLASS__, 'process_order_failed']);
            add_filter('woocommerce_email_order_meta_fields', [__CLASS__, 'add_codes_to_email_meta'], 10, 3);
            add_filter('woocommerce_email_content', [__CLASS__, 'replace_codes_placeholder'], 10, 4);
            add_action('admin_init', [__CLASS__, 'whatsapp_notification_settings']);
        }
    }

    public static function add_fields() {
        global $woocommerce, $post;
        echo '<div class="options_group">';

        woocommerce_wp_textarea_input([
            'id' => '_product_sequence_list',
            'label' => __('Sequências', 'woocommerce'),
            'placeholder' => __('Insira as sequências separadas por vírgula', 'woocommerce'),
            'desc_tip' => 'true',
            'description' => __('Adicione até 999 sequências separadas por vírgula.', 'woocommerce')
        ]);

        woocommerce_wp_textarea_input([
            'id' => '_email_body_text',
            'label' => __('Texto do E-mail', 'woocommerce'),
            'placeholder' => __('Insira o texto do e-mail. Use {codes} para inserir as sequências.', 'woocommerce'),
            'desc_tip' => 'true',
            'description' => __('Adicione o texto que será enviado por e-mail. Use {codes} para inserir as sequências.', 'woocommerce')
        ]);

        woocommerce_wp_text_input([
            'id' => '_webhook_url',
            'label' => __('URL do Webhook', 'woocommerce'),
            'placeholder' => __('Insira o URL do webhook', 'woocommerce'),
            'desc_tip' => 'true',
            'description' => __('Adicione o URL para enviar o webhook.', 'woocommerce')
        ]);

        echo '</div>';
    }

    public static function save_fields($post_id) {
        $product_sequence_list = $_POST['_product_sequence_list'];
        $email_body_text = $_POST['_email_body_text'];
        $webhook_url = $_POST['_webhook_url'];

        if (!empty($product_sequence_list)) {
            update_post_meta($post_id, '_product_sequence_list', sanitize_text_field($product_sequence_list));
        }

        if (!empty($email_body_text)) {
            update_post_meta($post_id, '_email_body_text', sanitize_text_field($email_body_text));
        }

        update_post_meta($post_id, '_webhook_url', sanitize_text_field($webhook_url));
    }

    public static function process_order($order_id) {
        $order = wc_get_order($order_id);
        $codes = [];

        foreach ($order->get_items() as $item_id => $item) {
            $product_id = $item->get_product_id();
            $sequences = get_post_meta($product_id, '_product_sequence_list', true);
            $email_body_text = get_post_meta($product_id, '_email_body_text', true);
            $webhook_url = get_post_meta($product_id, '_webhook_url', true);

            if (!empty($sequences)) {
                $sequences_array = explode(',', $sequences);
                $sequence_to_send = array_shift($sequences_array);
                update_post_meta($product_id, '_product_sequence_list', implode(',', $sequences_array));

                $codes[$item->get_name()] = $sequence_to_send;
            }
        }

        if (!empty($codes)) {
            self::send_sequence_email($order, $email_body_text, $codes);
            self::send_webhook($order, $webhook_url, $codes);
            self::send_whatsapp_message($order, $codes, 'completed');
            update_post_meta($order_id, '_codes', $codes);
        }
    }

    public static function process_order_failed($order_id) {
        $order = wc_get_order($order_id);
        $codes = [];

        foreach ($order->get_items() as $item_id => $item) {
            $product_id = $item->get_product_id();
            $sequences = get_post_meta($product_id, '_product_sequence_list', true);
            $sequence_to_send = explode(',', $sequences)[0];

            $codes[$item->get_name()] = $sequence_to_send;
        }

        if (!empty($codes)) {
            self::send_whatsapp_message($order, $codes, 'failed');
        }
    }

    public static function send_sequence_email($order, $email_body_text, $codes) {
        if (empty($email_body_text)) {
            $email_body_text = 'Obrigado pela sua compra. Aqui estão suas sequências:<br>{codes}';
        }

        $codes_text = '';
        foreach ($codes as $product => $code) {
            $codes_text .= $product . ': ' . $code . '<br>';
        }

        $email_body_text = str_replace('{codes}', $codes_text, $email_body_text);
        $email_body_text = nl2br($email_body_text);

        $to = $order->get_billing_email();
        $subject = 'Chaves de ativação';
        $body = $email_body_text;
        $headers = ['Content-Type: text/html; charset=UTF-8'];

        if (wp_mail($to, $subject, $body, $headers)) {
            error_log('E-mail enviado com sucesso para ' . $to);
        } else {
            error_log('Falha ao enviar e-mail para ' . $to);
        }
    }

    public static function send_webhook($order, $webhook_url, $codes) {
        if (empty($webhook_url)) {
            return;
        }

        $customer_name = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();
        $customer_email = $order->get_billing_email();
        $products = [];
        foreach ($order->get_items() as $item_id => $item) {
            $products[] = $item->get_name();
        }
        $product_list = implode(', ', $products);
        $order_total = $order->get_total();
        $order_status = $order->get_status();
        $customer_phone = $order->get_billing_phone();

        $webhook_data = [
            'customer_name' => $customer_name,
            'customer_email' => $customer_email,
            'products' => $product_list,
            'order_total' => $order_total,
            'order_id' => $order->get_id(),
            'order_status' => $order_status,
            'customer_phone' => $customer_phone,
            'sequences' => $codes
        ];

        $response = wp_remote_post($webhook_url, [
            'method' => 'POST',
            'body' => json_encode($webhook_data),
            'headers' => [
                'Content-Type' => 'application/json'
            ]
        ]);

        if (is_wp_error($response)) {
            error_log('Erro ao enviar webhook: ' . $response->get_error_message());
        }
    }

    public static function send_whatsapp_message($order, $codes, $status) {
        $codigo_pais = get_option('whatsapp_codigo_pais', '+55');
        $telefone_destinatario = preg_replace('/\D/', '', $order->get_billing_phone());
        $telefone_destinatario = $codigo_pais . $telefone_destinatario;

        $host = get_option('whatsapp_api_host');
        $connection_key = get_option('whatsapp_api_connection_key');
        $token = get_option('whatsapp_api_token');

        if (!$host || !$connection_key || !$token) {
            error_log('Configurações da API do WhatsApp não estão definidas corretamente.');
            return;
        }

        $api_url = "https://{$host}/message/sendText?connectionKey={$connection_key}";

        $mensagem_template = get_option('whatsapp_message_' . $status);
        if (empty($mensagem_template)) return;

        $mensagem = self::replace_placeholders($mensagem_template, $order, $codes);

        $data = [
            'phoneNumber' => $telefone_destinatario,
            'message' => $mensagem
        ];

        $response = wp_remote_post($api_url, [
            'method' => 'POST',
            'headers' => [
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . $token
            ],
            'body' => json_encode($data)
        ]);

        if (is_wp_error($response)) {
            error_log('Erro ao enviar mensagem pelo WhatsApp: ' . $response->get_error_message());
        }
    }

    public static function replace_placeholders($template, $order, $codes) {
        $placeholders = [
            '{nome_cliente}' => $order->get_billing_first_name(),
            '{email_cliente}' => $order->get_billing_email(),
            '{valor_total}' => $order->get_total(),
            '{itens_pedido}' => self::get_order_items_as_string($order),
            '{codes}' => self::get_codes_as_string($codes)
        ];

        foreach ($placeholders as $key => $value) {
            $template = str_replace($key, $value, $template);
        }

        return $template;
    }

    public static function get_order_items_as_string($order) {
        $items = [];
        foreach ($order->get_items() as $item_id => $item) {
            $items[] = $item->get_name();
        }
        return implode(', ', $items);
    }

    public static function get_codes_as_string($codes) {
        $codes_text = '';
        foreach ($codes as $product => $code) {
            $codes_text .= $product . ': ' . $code . "\n";
        }
        return $codes_text;
    }

    public static function add_codes_to_email_meta($fields, $sent_to_admin, $order) {
        $codes = get_post_meta($order->get_id(), '_codes', true);
        if (!empty($codes)) {
            $codes_text = self::get_codes_as_string($codes);
            $fields['codes'] = [
                'label' => __('Códigos do Produto'),
                'value' => nl2br($codes_text),
            ];
        }
        return $fields;
    }

    public static function replace_codes_placeholder($content, $email) {
        if ($email->id === 'customer_completed_order') {
            $order_id = $email->object->get_id();
            $codes = get_post_meta($order_id, '_codes', true);
            if (!empty($codes)) {
                $codes_text = self::get_codes_as_string($codes);
                $content = str_replace('{codes}', nl2br($codes_text), $content);
            }
        }
        return $content;
    }

    public static function whatsapp_notification_settings() {
        add_settings_section('whatsapp_api_settings_section', 'Configurações da API do WhatsApp', null, 'general');

        add_settings_field('whatsapp_api_host', 'Host da API do WhatsApp', [__CLASS__, 'display_text_field'], 'general', 'whatsapp_api_settings_section', ['name' => 'whatsapp_api_host']);
        register_setting('general', 'whatsapp_api_host');

        add_settings_field('whatsapp_api_connection_key', 'ConnectionKey da API do WhatsApp', [__CLASS__, 'display_text_field'], 'general', 'whatsapp_api_settings_section', ['name' => 'whatsapp_api_connection_key']);
        register_setting('general', 'whatsapp_api_connection_key');

        add_settings_field('whatsapp_api_token', 'Token da API do WhatsApp', [__CLASS__, 'display_text_field'], 'general', 'whatsapp_api_settings_section', ['name' => 'whatsapp_api_token']);
        register_setting('general', 'whatsapp_api_token');

        add_settings_field('whatsapp_codigo_pais', 'Código do país (WhatsApp)', [__CLASS__, 'display_text_field'], 'general', 'whatsapp_api_settings_section', ['name' => 'whatsapp_codigo_pais']);
        register_setting('general', 'whatsapp_codigo_pais');

        add_settings_field('whatsapp_message_completed', 'Mensagem para Pedido Concluído', [__CLASS__, 'display_textarea_field'], 'general', 'whatsapp_api_settings_section', ['name' => 'whatsapp_message_completed']);
        register_setting('general', 'whatsapp_message_completed');

        add_settings_field('whatsapp_message_failed', 'Mensagem para Pedido Mal Sucedido', [__CLASS__, 'display_textarea_field'], 'general', 'whatsapp_api_settings_section', ['name' => 'whatsapp_message_failed']);
        register_setting('general', 'whatsapp_message_failed');
    }

    public static function display_text_field($args) {
        $name = $args['name'];
        $value = get_option($name, '');
        echo "<input type='text' name='$name' value='$value' class='regular-text'>";
    }

    public static function display_textarea_field($args) {
        $name = $args['name'];
        $value = get_option($name, '');
        echo "<textarea name='$name' rows='5' class='large-text'>$value</textarea>";
    }
}

add_action('plugins_loaded', ['WooCommerce_Combined_Plugin', 'init']);